@@ -59,6 +59,8 @@ void BamfLauncherIcon::ActivateLauncherIcon(ActionArg arg)
 {
   SimpleLauncherIcon::ActivateLauncherIcon(arg);
   bool scaleWasActive = WindowManager::Default()->IsScaleActive();
+  GList    *l;
+  BamfView *view;
 
   bool active, running;
   active = bamf_view_is_active(BAMF_VIEW(m_App));
@@ -70,6 +72,34 @@ void BamfLauncherIcon::ActivateLauncherIcon(ActionArg arg)
     return;
   }
 
+  /* We should check each child to see if there is
+   * an unmapped (!= minimized) window around and
+   * if so force "Focus" behaviour */
+
+  if (arg.source != ActionArg::SWITCHER)
+  {
+    bool any_visible = false;
+    for (l = bamf_view_get_children(BAMF_VIEW(m_App)); l; l = l->next)
+    {
+      view = static_cast <BamfView*> (l->data);
+
+      if (BAMF_IS_WINDOW(view))
+      {
+        Window xid = bamf_window_get_xid(BAMF_WINDOW(view));
+
+        if (WindowManager::Default ()->IsWindowOnCurrentDesktop(xid))
+          any_visible = true;
+        if (!WindowManager::Default ()->IsWindowMapped(xid))
+        {
+          active = false;
+        }
+      }
+    }
+
+    if (!any_visible)
+      active = false;
+  }
+
   /* Behaviour:
    * 1) Nothing running -> launch application
    * 2) Running and active -> spread application
@@ -98,7 +128,7 @@ void BamfLauncherIcon::ActivateLauncherIcon(ActionArg arg)
       if (scaleWasActive) // #5 above
       {
         WindowManager::Default()->TerminateScale();
-        Focus();
+        Focus(arg);
       }
       else // #2 above
       {
@@ -111,13 +141,13 @@ void BamfLauncherIcon::ActivateLauncherIcon(ActionArg arg)
       if (scaleWasActive) // #4 above
       {
         WindowManager::Default()->TerminateScale();
-        Focus();
+        Focus(arg);
         if (arg.source != ActionArg::SWITCHER)
           Spread(0, false);
       }
       else // #3 above
       {
-        Focus();
+        Focus(arg);
       }
     }
   }
@@ -257,7 +287,7 @@ std::vector<Window> BamfLauncherIcon::RelatedXids ()
   children = bamf_view_get_children(BAMF_VIEW(m_App));
   for (l = children; l; l = l->next)
   {
-    view = (BamfView*) l->data;
+    view = static_cast <BamfView*> (l->data);
     if (BAMF_IS_WINDOW(view))
     {
       guint32 xid = bamf_window_get_xid(BAMF_WINDOW(view));
@@ -280,7 +310,7 @@ std::string BamfLauncherIcon::NameForWindow (Window window)
   children = bamf_view_get_children(BAMF_VIEW(m_App));
   for (l = children; l; l = l->next)
   {
-    view = (BamfView*) l->data;
+    view = static_cast <BamfView*> (l->data);
     if (BAMF_IS_WINDOW(view) && (Window) bamf_window_get_xid(BAMF_WINDOW(view)) == window)
     {
       gchar *name = bamf_view_get_name (view);
@@ -308,8 +338,9 @@ void BamfLauncherIcon::OnWindowMinimized(guint32 xid)
   UpdateQuirkTimeDelayed(300, QUIRK_SHIMMER);
 }
 
-gboolean BamfLauncherIcon::OnWindowMovedTimeout(BamfLauncherIcon* self)
+gboolean BamfLauncherIcon::OnWindowMovedTimeout(gpointer data)
 {
+  BamfLauncherIcon *self = static_cast <BamfLauncherIcon *> (data);
   GList *children = bamf_view_get_children(BAMF_VIEW(self->m_App));
 
   bool any_on_current = false;
@@ -439,7 +470,7 @@ void BamfLauncherIcon::AddProperties(GVariantBuilder* builder)
   int i = 0;
   for (l = children; l; l = l->next)
   {
-    view = (BamfView*) l->data;
+    view = static_cast <BamfView*> (l->data);
 
     if (BAMF_IS_WINDOW(view))
     {
@@ -462,7 +493,7 @@ bool BamfLauncherIcon::OwnsWindow(Window w)
 
   for (l = children; l; l = l->next)
   {
-    view = (BamfView*) l->data;
+    view = static_cast <BamfView*> (l->data);
 
     if (BAMF_IS_WINDOW(view))
     {
@@ -536,11 +567,12 @@ void BamfLauncherIcon::OpenInstanceLauncherIcon(ActionArg arg)
   ubus_server_send_message(ubus_server_get_default(), UBUS_LAUNCHER_ACTION_DONE, NULL);
 }
 
-void BamfLauncherIcon::Focus()
+void BamfLauncherIcon::Focus(ActionArg arg)
 {
   GList* children, *l;
   BamfView* view;
   bool any_urgent = false;
+  bool any_visible = false;
 
   children = bamf_view_get_children(BAMF_VIEW(m_App));
 
@@ -549,13 +581,17 @@ void BamfLauncherIcon::Focus()
   /* get the list of windows */
   for (l = children; l; l = l->next)
   {
-    view = (BamfView*) l->data;
+    view = static_cast <BamfView*> (l->data);
 
     if (BAMF_IS_WINDOW(view))
     {
       Window xid = bamf_window_get_xid(BAMF_WINDOW(view));
       bool urgent = bamf_view_is_urgent(view);
 
+      if (WindowManager::Default ()->IsWindowOnCurrentDesktop (xid) &&
+          WindowManager::Default ()->IsWindowVisible (xid))
+        any_visible = true;
+
       if (any_urgent)
       {
         if (urgent)
@@ -566,15 +602,31 @@ void BamfLauncherIcon::Focus()
         if (urgent)
         {
           windows.clear();
+          any_visible = false;
           any_urgent = true;
         }
-        windows.push_back(xid);
       }
+      windows.push_back(xid);
     }
   }
 
   g_list_free(children);
-  WindowManager::Default()->FocusWindowGroup(windows);
+  if (arg.source != ActionArg::SWITCHER)
+  {
+    if (any_visible)
+    {
+      WindowManager::Default()->FocusWindowGroup(windows,
+       WindowManager::FocusVisibility::ForceUnminimizeInvisible);
+    }
+    else
+    {
+      WindowManager::Default()->FocusWindowGroup(windows,
+       WindowManager::FocusVisibility::ForceUnminimizeOnCurrentDesktop);
+    }
+  }
+  else
+    WindowManager::Default()->FocusWindowGroup(windows,
+      WindowManager::FocusVisibility::OnlyVisible);
 }
 
 bool BamfLauncherIcon::Spread(int state, bool force)
@@ -586,7 +638,7 @@ bool BamfLauncherIcon::Spread(int state, bool force)
   std::vector<Window> windowList;
   for (l = children; l; l = l->next)
   {
-    view = (BamfView*) l->data;
+    view = static_cast <BamfView*> (l->data);
 
     if (BAMF_IS_WINDOW(view))
     {
@@ -601,7 +653,7 @@ bool BamfLauncherIcon::Spread(int state, bool force)
 
 void BamfLauncherIcon::OnClosed(BamfView* view, gpointer data)
 {
-  BamfLauncherIcon* self = (BamfLauncherIcon*) data;
+  BamfLauncherIcon* self = static_cast <BamfLauncherIcon*> (data);
 
   if (!bamf_view_is_sticky(BAMF_VIEW(self->m_App)))
     self->Remove();
@@ -610,7 +662,7 @@ void BamfLauncherIcon::OnClosed(BamfView* view, gpointer data)
 void BamfLauncherIcon::OnUserVisibleChanged(BamfView* view, gboolean visible,
                                             gpointer data)
 {
-  BamfLauncherIcon* self = (BamfLauncherIcon*) data;
+  BamfLauncherIcon* self = static_cast <BamfLauncherIcon*> (data);
 
   if (!bamf_view_is_sticky(BAMF_VIEW(self->m_App)))
     self->SetQuirk(QUIRK_VISIBLE, visible);
@@ -619,7 +671,7 @@ void BamfLauncherIcon::OnUserVisibleChanged(BamfView* view, gboolean visible,
 void BamfLauncherIcon::OnRunningChanged(BamfView* view, gboolean running,
                                         gpointer data)
 {
-  BamfLauncherIcon* self = (BamfLauncherIcon*) data;
+  BamfLauncherIcon* self = static_cast <BamfLauncherIcon*> (data);
   self->SetQuirk(QUIRK_RUNNING, running);
 
   if (running)
@@ -631,13 +683,13 @@ void BamfLauncherIcon::OnRunningChanged(BamfView* view, gboolean running,
 
 void BamfLauncherIcon::OnActiveChanged(BamfView* view, gboolean active, gpointer data)
 {
-  BamfLauncherIcon* self = (BamfLauncherIcon*) data;
+  BamfLauncherIcon* self = static_cast <BamfLauncherIcon*> (data);
   self->SetQuirk(QUIRK_ACTIVE, active);
 }
 
 void BamfLauncherIcon::OnUrgentChanged(BamfView* view, gboolean urgent, gpointer data)
 {
-  BamfLauncherIcon* self = (BamfLauncherIcon*) data;
+  BamfLauncherIcon* self = static_cast <BamfLauncherIcon*> (data);
   self->SetQuirk(QUIRK_URGENT, urgent);
 }
 
@@ -671,7 +723,7 @@ void BamfLauncherIcon::EnsureWindowState()
 
 void BamfLauncherIcon::OnChildAdded(BamfView* view, BamfView* child, gpointer data)
 {
-  BamfLauncherIcon* self = (BamfLauncherIcon*) data;
+  BamfLauncherIcon* self = static_cast <BamfLauncherIcon*> (data);
   self->EnsureWindowState();
   self->UpdateMenus();
   self->UpdateIconGeometries(self->GetCenter());
@@ -679,7 +731,7 @@ void BamfLauncherIcon::OnChildAdded(BamfView* view, BamfView* child, gpointer da
 
 void BamfLauncherIcon::OnChildRemoved(BamfView* view, BamfView* child, gpointer data)
 {
-  BamfLauncherIcon* self = (BamfLauncherIcon*) data;
+  BamfLauncherIcon* self = static_cast <BamfLauncherIcon*> (data);
   self->EnsureWindowState();
 }
 
@@ -804,7 +856,7 @@ void BamfLauncherIcon::OnQuit(DbusmenuMenuitem* item, int time, BamfLauncherIcon
 
   for (l = children; l; l = l->next)
   {
-    view = (BamfView*) l->data;
+    view = static_cast <BamfView*> (l->data);
 
     if (BAMF_IS_WINDOW(view))
     {
@@ -915,7 +967,7 @@ static void OnAppLabelActivated(DbusmenuMenuitem* sender,
   if (!data)
     return;
 
-  self = (BamfLauncherIcon*) data;
+  self = static_cast <BamfLauncherIcon*> (data);
   self->ActivateLauncherIcon(ActionArg(ActionArg::OTHER, 0));
 }
 
@@ -1088,7 +1140,7 @@ void BamfLauncherIcon::UpdateIconGeometries(nux::Point3 center)
 
   for (l = children; l; l = l->next)
   {
-    view = (BamfView*) l->data;
+    view = static_cast <BamfView*> (l->data);
 
     if (BAMF_IS_WINDOW(view))
     {
@@ -1147,7 +1199,7 @@ std::set<std::string> BamfLauncherIcon::ValidateUrisForLaunch(unity::DndData& ur
 
 gboolean BamfLauncherIcon::OnDndHoveredTimeout(gpointer data)
 {
-  BamfLauncherIcon* self = (BamfLauncherIcon*) data;
+  BamfLauncherIcon* self = static_cast <BamfLauncherIcon*> (data);
   
   // for now, let's not do this, it turns out to be quite buggy
   //if (self->_dnd_hovered && bamf_view_is_running(BAMF_VIEW(self->m_App)))
@@ -1220,7 +1272,7 @@ BamfLauncherIcon::SwitcherPriority()
   /* get the list of windows */
   for (l = children; l; l = l->next)
   {
-    view = (BamfView*) l->data;
+    view = static_cast <BamfView*> (l->data);
 
     if (BAMF_IS_WINDOW(view))
     {
@@ -1245,7 +1297,7 @@ BamfLauncherIcon::GetSupportedTypes()
 gboolean
 BamfLauncherIcon::FillSupportedTypes(gpointer data)
 {
-  BamfLauncherIcon* self = (BamfLauncherIcon*) data;
+  BamfLauncherIcon* self = static_cast <BamfLauncherIcon*> (data);
   
   if (self->_fill_supported_types_id)
   {
