@@ -119,6 +119,7 @@ PanelMenuView::PanelMenuView(int padding)
   _panel_titlebar_grab_area = new PanelTitlebarGrabArea();
   _panel_titlebar_grab_area->SetParentObject(this);
   _panel_titlebar_grab_area->SinkReference();
+  _panel_titlebar_grab_area->mouse_down.connect(sigc::mem_fun(this, &PanelMenuView::OnMouseClicked));
   _panel_titlebar_grab_area->mouse_down.connect(sigc::mem_fun(this, &PanelMenuView::OnMouseMiddleClicked));
   _panel_titlebar_grab_area->mouse_down.connect(sigc::mem_fun(this, &PanelMenuView::OnMaximizedGrabStart));
   _panel_titlebar_grab_area->mouse_drag.connect(sigc::mem_fun(this, &PanelMenuView::OnMaximizedGrabMove));
@@ -447,7 +448,7 @@ PanelMenuView::Draw(nux::GraphicsEngine& GfxContext, bool force_draw)
       lockrect.pBits = 0;
       bool locked = false;
 
-      if (_gradient_texture.IsNull())
+      if (_gradient_texture.IsNull() || (_gradient_texture->GetWidth() != geo.width))
       {
         build_gradient = true;
       }
@@ -1258,6 +1259,7 @@ PanelMenuView::GetMaximizedWindow()
 void
 PanelMenuView::OnMaximizedGrabStart(int x, int y, unsigned long button_flags, unsigned long)
 {
+  Window maximized_win;
   if (nux::GetEventButton(button_flags) != 1 || _places_showing)
     return;
 
@@ -1266,8 +1268,15 @@ PanelMenuView::OnMaximizedGrabStart(int x, int y, unsigned long button_flags, un
   //
   // This is a workaround to avoid that the grid plugin would be fired
   // showing the window shape preview effect. See bug #838923
-  if (GetMaximizedWindow() != 0)
+
+  maximized_win = GetMaximizedWindow ();
+
+  if (maximized_win != 0)
+  {
+    /* Always activate the window in case it is on another monitor */
+    WindowManager::Default ()->Activate (maximized_win);
     _panel_titlebar_grab_area->SetGrabbed(true);
+  }
 }
 
 void
@@ -1327,7 +1336,7 @@ PanelMenuView::OnMaximizedGrabEnd(int x, int y, unsigned long, unsigned long)
 void
 PanelMenuView::OnMouseDoubleClicked(int x, int y, unsigned long button_flags, unsigned long)
 {
-  if (nux::GetEventButton(button_flags) != 1)
+  if (nux::GetEventButton(button_flags) != 1 || _places_showing)
     return;
 
   guint32 window_xid = GetMaximizedWindow();
@@ -1339,10 +1348,24 @@ PanelMenuView::OnMouseDoubleClicked(int x, int y, unsigned long button_flags, un
   }
 }
 
+void
+PanelMenuView::OnMouseClicked(int x, int y, unsigned long button_flags, unsigned long)
+{
+  if (nux::GetEventButton(button_flags) != 1 || _places_showing)
+    return;
+
+  guint32 window_xid = GetMaximizedWindow();
+
+  if (window_xid != 0)
+  {
+    WindowManager::Default()->Raise(window_xid);
+  }
+}
+
 void
 PanelMenuView::OnMouseMiddleClicked(int x, int y, unsigned long button_flags, unsigned long)
 {
-  if (nux::GetEventButton(button_flags) != 2)
+  if (nux::GetEventButton(button_flags) != 2 || _places_showing)
     return;
 
   guint32 window_xid = GetMaximizedWindow();
@@ -1357,18 +1380,17 @@ PanelMenuView::OnMouseMiddleClicked(int x, int y, unsigned long button_flags, un
 const gchar*
 PanelMenuView::GetName()
 {
-  return "MenuView";
+  return NULL;
 }
 
 const gchar*
 PanelMenuView::GetChildsName()
 {
-  return "entries";
+  return NULL;
 }
 
 void PanelMenuView::AddProperties(GVariantBuilder* builder)
 {
-  variant::BuilderWrapper(builder).add(GetGeometry());
 }
 
 /*
