@@ -136,6 +136,7 @@ PanelView::~PanelView()
 {
   if (_track_menu_pointer_id)
     g_source_remove(_track_menu_pointer_id);
+
   _style->UnReference();
   UBusServer *ubus = ubus_server_get_default();
   ubus_server_unregister_interest(ubus, _handle_bg_color_update);
@@ -143,6 +144,9 @@ PanelView::~PanelView()
   ubus_server_unregister_interest(ubus, _handle_dash_shown);
   _on_indicator_updated_connections.clear();
 
+  indicator::EntryLocationMap locations;
+  _remote->SyncGeometries(GetName() + boost::lexical_cast<std::string>(_monitor), locations);
+
   delete _bg_layer;
 }
 
@@ -272,7 +276,7 @@ PanelView::Draw(nux::GraphicsEngine& GfxContext, bool force_draw)
 
   GfxContext.PopClippingRectangle();
 
-  if (_needs_geo_sync && _menu_view->GetControlsActive())
+  if (_needs_geo_sync)
   {
     SyncGeometries();
     _needs_geo_sync = false;
@@ -600,8 +604,7 @@ PanelView::SetOpacity(float opacity)
 
   _opacity = opacity;
 
-  if (_opacity < 1.0f && !_dash_is_open)
-    bg_effect_helper_.enabled = false;
+  bg_effect_helper_.enabled = (_opacity < 1.0f || _dash_is_open);
 
   ForceUpdateBackground();
 }
@@ -622,9 +625,13 @@ void
 PanelView::SyncGeometries()
 {
   indicator::EntryLocationMap locations;
-  _menu_view->GetGeometryForSync(locations);
+  std::string panel_id = GetName() + boost::lexical_cast<std::string>(_monitor);
+
+  if (_menu_view->GetControlsActive())
+    _menu_view->GetGeometryForSync(locations);
+
   _indicators->GetGeometryForSync(locations);
-  _remote->SyncGeometries(GetName(), locations);
+  _remote->SyncGeometries(panel_id, locations);
 }
 
 void
